macroScript Rivals_NR_import
category:"Game file import"
tooltip:"NFS Rivals - NinjaRipper import"
buttontext:"Rivals NR"
autoUndoEnabled:false
(
	struct vertexProperty (name, index, offset, size, elements = #())
	
	rivals_rip = undefined
	framePath = undefined
	matcount = 0
	
	fn importRip ripFile =
	(
		vPropArr = #()
		farr = #()
		varr = #()
		narr = #()
		t1arr = #()
		t2arr = #()
		biarr = #()
		biOffset = 0
		
		f = fopen ripFile "rb"
		
		fseek f 8 #seek_set
		numFaces = readlong f #unsigned
		numVerts = readlong f #unsigned
		stride = readlong f #unsigned
		
		if (stride != 128 and stride != 120) do
		(
			--format "Unknown vertex stride: %. % was not imported\n" stride (getFilenameFile ripFile)
			return undefined
		)
		
		numTextures = readlong f #unsigned
		numShaders = readlong f #unsigned
		numProps = readlong f #unsigned
		
		for i=1 to numProps do
		(
			vProp = vertexProperty()
			vProp.name = readstring f
			vProp.index = readlong f #unsigned
			vProp.offset = readlong f #unsigned
			vProp.size = readlong f #unsigned
			vProp.elements.count = readlong f #unsigned
			
			for j=1 to vProp.elements.count do vProp.elements[j] = readlong f #unsigned
				--0 float
				--1 uint32
				--2 sint32
			append vPropArr vProp
			-- remember bone index offset
			if (vProp.elements[1] == 1 and vProp.offset >= 96) do biOffset = vProp.offset - 96
		)
		strideEnd = stride - 112 - biOffset
		
		format "Importing %\n" ripFile
		
		texarr = #()
		for i=1 to numTextures do append texarr (readstring f)
		
		for i=1 to numShaders do ashad = readstring f
		
		
		for i=1 to numFaces do
		(
			ind1 = readlong f #unsigned
			ind2 = readlong f #unsigned
			ind3 = readlong f #unsigned
			append farr [ind1+1, ind2+1, ind3+1]
		)
		
		for v=1 to numVerts do
		(
			--NFS Rivals	
			vx = readFloat f
			vy = readFloat f
			vz = readFloat f
			vw = readFloat f
			append varr [vx, -vz, vy]
			
			fseek f 32 #seek_cur
			
			nx = readFloat f
			ny = readFloat f
			nz = readFloat f
			nw = readFloat f
			append narr [nx, -nz, ny]
			
			fseek f 16 #seek_cur --tangents
			
			tx = readFloat f
			ty = readFloat f
			tz = readFloat f
			tw = readFloat f
			append t1arr [tx, 1-ty, 0]
			append t2arr [tz, 1-tw, 0]
			
			fseek f biOffset #seek_cur
			
			b1 = readFloat f
			b2 = readFloat f
			b3 = readFloat f
			b4 = readFloat f
			append biarr [b1, b2, b3, b4]
			
			fseek f strideEnd #seek_cur
		)
		
		fclose f
		
		amesh = mesh vertices:varr faces:farr name:(getFilenameFile ripFile)
		setUserProp amesh "imported_with" "custom Ninja Ripper script by Chipicao"
		
		--double switch UVs on non-dirt parts
		if (((finditem texarr "Tex_0000_6.dds") == 0) and ((makeuniquearray t2arr).count == 1)) do
		(
			tarr = t1arr
			t1arr = t2arr
			t2arr = tarr
		)
		
		meshop.setMapSupport amesh 1 true
		for v = 1 to numVerts do meshop.setMapVert amesh 1 v t2Arr[v]
		
		meshop.setNumMaps amesh 3 keep:true
		meshop.setMapSupport amesh 2 true
		for v = 1 to numVerts do meshop.setMapVert amesh 2 v t1Arr[v]
		
		if rivals_rip.iNormals.checked do
		(
			--set smoothing group of all faces to 1 to get one normal per vertex
			for face = 1 to amesh.numfaces do setFaceSmoothGroup amesh face 1
			--set normals via edit normals modifier
			select amesh
			addmodifier amesh (Edit_Normals ()) ui:off
			amesh.Edit_Normals.MakeExplicit selection:#{1..numVerts}
			EN_convertVS = amesh.Edit_Normals.ConvertVertexSelection
			EN_setNormal = amesh.Edit_Normals.SetNormal
			normID = #{}
			--apply normals
			for v = 1 to numVerts do
			(
				free normID
				EN_convertVS #{v} &normID
				for id in normID do EN_setNormal id nArr[v]
			)
			collapseStack amesh
		)
		
		--apply material and textures
		if rivals_rip.applytex.checked do
		(
			draw = (filterstring (getFilenameFile ripFile) "_.")[2]
			amaterial = Standard showinviewport:true name:("Material_" + draw)
			amaterial.specularLevel = 30
			amaterial.bumpMapAmount = 100
			matcount +=1
			atexture = framePath + "Tex_" + draw + "_"
			if doesfileexist (atexture + "2.dds") do amaterial.diffusemap = BitmapTexture filename:(atexture + "2.dds")
			
			if doesfileexist (atexture + "3.dds") do
			(
				amaterial.specularMap = ColorCorrection map:(BitmapTexture filename:(atexture + "3.dds")) rewireG:0 rewireB:0 --Red
				amaterial.specularLevelMap = ColorCorrection map:(BitmapTexture filename:(atexture + "3.dds")) rewireR:1 rewireB:1 --Green
			)
			
			if doesfileexist (atexture + "4.dds") then
			(
				amaterial.bumpMap = Normal_Bump normal_map:(BitmapTexture filename:(atexture + "4.dds"))
			)
			else
			(
				if doesfileexist (atexture + "5.dds") then
				(
					amaterial.bumpMap = Normal_Bump normal_map:(BitmapTexture filename:(atexture + "5.dds"))
				)
				else
				(
					if doesfileexist (atexture + "7.dds") do amaterial.bumpMap = Normal_Bump normal_map:(BitmapTexture filename:(atexture + "7.dds"))
				)
			)
			if matcount <25 do meditmaterials[matcount] = amaterial
			
			amesh.material = amaterial
		)
		
		--slice model
		if rivals_rip.slicemodel.checked do
		(
			bonez = #()
			vertsel = #()
			for b=1 to biarr.count do
			(
				ab = biarr[b][1]
				index = finditem bonez ab
				if index !=0 then append vertsel[index] b
				else
				(
					append bonez ab
					append vertsel #{b}
				)
			)
			
			if bonez.count >1 do
			(
				for b=1 to bonez.count-1 do
				(
					bmesh = copy amesh
					bmesh.selectedVerts = #{1..bmesh.numverts} - vertsel[b]
					delete bmesh.selectedverts
					bmesh.name = amesh.name + "_" + (b as string)
					setUserProp bmesh "imported_with" "custom Ninja Ripper script by Chipicao"
				)
				
				amesh.selectedVerts = #{1..amesh.numverts} - vertsel[bonez.count]
				delete amesh.selectedverts
				amesh.name += "_" + (bonez.count as string)
			)
		)
	)
	
	rollout rivals_rip "NFS Rivals - NinjaRipper import 0.5" width:240 height:140
	(
		checkbox iNormals "Import normals" pos:[16,16] width:96 height:16 checked:true tooltip:"Disable normal import to speed up the process.\nAlways enable for proper import!"
		checkbox slicemodel "Slice model" pos:[128,16] width:96 height:16 checked:true tooltip:"Slice the model according to animations.\nThis should split moving parts such as doors and wheels."
		checkbox applytex "Try to apply materials and textures" pos:[16,40] width:200 height:16 checked:true tooltip:"Attempt to assign textures automatically.\nWorks in 90% of cases"
		
		button iFile "Load file" pos:[16,68] width:96 height:24 tooltip:"Load a single rip file"
		button iFolder "Load folder" pos:[128,68] width:96 height:24 tooltip:"Load all rip files inside a folder"
		button switchUV "Swap UV1 and UV2 on selected meshes" pos:[16,104] width:208 height:26 tooltip:"Use this after import if needed.\nMost cockpit materials will need swapping."
		--label fuckEmAll "Fuck those who sell models they didn't make!" pos:[16,140]
		
		on iFile pressed do
		(
			ripFile = getOpenFileName types:"NFS Rivals NinjaRipper .rip|*.rip"
			if ripFile != undefined do with redraw off
			(
				max modify mode -- open mod panel for edit_normals to work
				cui.expertModeOn() --hide panel for faster processing
				--disableSceneRedraw()
				
				framePath = getFilenamePath ripFile
				importRip ripFile
				
				cui.expertModeOff()
				--enableSceneRedraw()
				max select none
				max tool zoomextents all
				max tool zoomextents all
			)
		)
		
		on iFolder pressed do
		(
			framePath = getSavePath caption:"Select Frame rip folder"
			
			if framepath != undefined do with redraw off
			(
				max modify mode -- open mod panel for edit_normals to work
				cui.expertModeOn() --hide panel for faster processing
				--disableSceneRedraw()
				
				framePath += "\\"
				ripFiles = getFiles (framePath + "*.rip")
				
				for ripFile in ripFiles do importRip ripFile
				
				cui.expertModeOff()
				--enableSceneRedraw()
				max select none
				max tool zoomextents all
				max tool zoomextents all
			)
		)
		
		on switchUV pressed do
		(
			for amesh in selection where classof amesh == Editable_mesh do
			(
				channelInfo.CopyChannel amesh 3 2
				channelInfo.PasteChannel amesh 3 3
				
				channelInfo.CopyChannel amesh 3 1
				channelInfo.PasteChannel amesh 3 2
				
				channelInfo.CopyChannel amesh 3 3
				channelInfo.PasteChannel amesh 3 1
				
				channelInfo.ClearChannel amesh 3
				
				maxOps.CollapseNodeTo amesh 1 off
			)
			forceCompleteRedraw() --for some reason this doesn;t work
		)
	)
	createDialog rivals_rip
)